@@ -32,21 +32,6 @@ logger = logging.getLogger()
 _CACHED_LOCATIONS = {}  # TODO: Implement a smarter solution for caching project's location
 
 
-def _get_project_default_location(credentials, project_id: str, default_zone: str = '1') -> str:
-    location = _CACHED_LOCATIONS.get(project_id, None)
-
-    if not location:
-        service = build(serviceName='appengine', version='v1', credentials=credentials, cache_discovery=False)
-        data = service.apps().get(appsId=project_id).execute()
-        location = data['locationId']
-        try:
-            int(location[-1])
-        except ValueError:
-            location = data['locationId'] + default_zone
-        _CACHED_LOCATIONS[project_id] = location
-    return location
-
-
 MINIMAL_SCOPES = [
     'https://www.googleapis.com/auth/cloud-platform',
 ]
@@ -66,18 +51,28 @@ class GoogleCloudPilotAPI(abc.ABC):
             location: str = None,
             project_id: str = None,
             impersonate_account: str = None,
+            credentials: Credentials = None,
             **kwargs,
     ):
-        self.credentials, credential_project_id = self._set_credentials(
-            subject=subject,
-            impersonate_account=impersonate_account or DEFAULT_SERVICE_ACCOUNT,
-        )
+        if credentials:
+            self.credentials, credential_project_id = credentials, getattr(credentials, 'project_id', None)
+        else:
+            self.credentials, credential_project_id = self._set_credentials(
+                subject=subject,
+                impersonate_account=impersonate_account or DEFAULT_SERVICE_ACCOUNT,
+            )
         self.project_id = self._set_project_id(project_id=project_id, credential_project_id=credential_project_id)
-        self.location = self._set_location(location=location)
 
         self.client = self._build_client(**kwargs)
 
+        self.location = self._set_location(location=location)
+
+    def _get_client_extra_kwargs(self):
+        return {}
+
     def _build_client(self, **kwargs) -> Union[Resource, _client_class]:
+        kwargs.update(self._get_client_extra_kwargs())
+
         return (self._client_class or build)(
             credentials=self.credentials,
             **kwargs
@@ -87,10 +82,7 @@ class GoogleCloudPilotAPI(abc.ABC):
         return project_id or DEFAULT_PROJECT or credential_project_id
 
     def _set_location(self, location: str = None) -> str:
-        return location or DEFAULT_LOCATION or _get_project_default_location(
-            credentials=self.credentials,
-            project_id=self.project_id,
-        )
+        return location or DEFAULT_LOCATION or self._get_project_default_location()
 
     @classmethod
     def _impersonate_account(
@@ -158,7 +150,7 @@ class GoogleCloudPilotAPI(abc.ABC):
         else:
             credentials, project_id = cls._cached_credentials
 
-        if impersonate_account:
+        if impersonate_account and getattr(credentials, 'service_account_email') != impersonate_account:
             credentials, impersonated_project_id = cls._impersonate_account(
                 credentials=credentials,
                 service_account=impersonate_account,
@@ -169,7 +161,7 @@ class GoogleCloudPilotAPI(abc.ABC):
         if subject:
             credentials = cls._delegated_credential(credentials=credentials, subject=subject, scopes=all_scopes)
 
-        return credentials, (project_id or getattr(credentials, 'project_id'))
+        return credentials, (project_id or getattr(credentials, 'project_id', None))
 
     @property
     def oidc_token(self) -> Dict[str, Dict[str, str]]:
@@ -204,7 +196,22 @@ class GoogleCloudPilotAPI(abc.ABC):
         return project['projectNumber']
 
     def _as_duration(self, seconds):
-        return Duration(seconds=seconds)
+        return Duration(seconds=seconds) if seconds else None
+
+    @classmethod
+    def build_from(cls, client: 'GoogleCloudPilotAPI', project_id: str = None):
+        return cls(
+            credentials=client.credentials,
+            project_id=project_id or client.project_id,
+        )
+
+    def _get_project_default_location(self, project_id: str = None) -> Union[str, None]:
+        from gcp_pilot.app_engine import AppEngine  # pylint: disable=import-outside-toplevel
+        try:
+            app_engine = AppEngine.build_from(client=self, project_id=project_id)
+            return app_engine.location
+        except exceptions.NotFound:
+            return None
 
 
 class AccountManagerMixin:
@@ -251,7 +258,7 @@ class AppEngineBasedService:
     # So these clients cannot just choose a desired region to work on, they must use the App Engine's
     # previously chosen region.
     def _set_location(self, location: str = None):
-        project_location = _get_project_default_location(credentials=self.credentials, project_id=self.project_id)
+        project_location = self._get_project_default_location()
 
         explicit_location = location
         if explicit_location and explicit_location != project_location:
